import { BrowserModule } from '@angular/platform-browser';
import { NgModule, APP_INITIALIZER } from '@angular/core';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { AppComponent } from './app.component';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { HttpClientModule } from '@angular/common/http';
import { RouterModule } from '@angular/router';
import { AppRoutingModule } from './app.routing.module';
import { AppServices, AppConfigService } from './services';
import { AppComponents } from './components';
import { UserModule } from './user/user.module';
import { UserService } from './user/services';

// Function used by APP_INITIALIZER before the app start: init user info / statut (expect a promise)
export function initUserService(authService: UserService) {
  return (): Promise<any> => {
    return new Promise((resolve, reject) => {
      authService.initializeService();
      resolve();
    });
  };
}

export function initAppConfig(appConfigService: AppConfigService) {
  return (): Promise<any> => {
    return new Promise((resolve, reject) => {
      appConfigService.load();
      resolve();
    });
  };
}

@NgModule({
  declarations: [
    AppComponent,
    ...AppComponents,
  ],
  imports: [
    AppRoutingModule,
    BrowserModule,
    FormsModule,
    RouterModule,
    BrowserAnimationsModule,
    HttpClientModule,
    ReactiveFormsModule,
    UserModule,
  ],
  providers: [
    ...AppServices,
    {
      provide: APP_INITIALIZER,
      useFactory: initUserService,
      deps: [UserService],
      multi: true,
    },
    {
      provide: APP_INITIALIZER,
      useFactory: initAppConfig,
      deps: [AppConfigService],
      multi: true,
    },
  ],
  bootstrap: [AppComponent],
})
export class AppModule { }
