import { Component, OnInit } from '@angular/core';
import { FormatService } from 'src/app/services/format.service';
import { Subscription } from 'rxjs';
import { PaginatorOptions } from 'src/app/models/paginator-options.model';
import { Format, FormatRO } from 'src/app/models/format.model';

@Component({
  selector: 'app-formats',
  templateUrl: './formats.component.html',
  styleUrls: ['./formats.component.scss'],
})
export class FormatsComponent implements OnInit {

  formats: Format[];
  searchChangeSub: Subscription;

  // Paginator options
  paginator: PaginatorOptions;

  sortValue: string;

  totalElement: number;
  pageSize = 10;
  pageSizeOptions = [5, 10, 25, 100];
  filters = {
    name: '',
  };
  where = {};

  constructor(
    private formatsService: FormatService,
  ) {
    this.paginator = {
      pageIndex: this.formatsService.pageNumber,
      length: 0,
      limit: this.formatsService.limit,
      pageSizeOptions: [5, 10, 20],
    };
  }

  ngOnInit(): void {
    this.formatsService.sortOptions = {
      value: 'name',
      order: 'asc',
    };
    this.search();

    this.searchChangeSub = this.formatsService.searchChange$.subscribe(
      () => {
        this.search();
      },
    );
  }

  private search() {
    this.formatsService.getFormats()
      .subscribe((items: FormatRO) => {
        this.formats = items.formats;
        this.totalElement = items.totalCount;

        this.paginator.limit = this.formatsService.limit;
        this.paginator.pageIndex = this.formatsService.pageNumber;
        this.paginator.length = items.totalCount;
      });
  }

  // When pagination is changed by user, we update datasetList with new pagination options
  changePagination(pageIndex) {
    this.formatsService.paginationChanged(this.paginator.limit, pageIndex);
  }

  changePageSize(pageSize) {
    this.formatsService.paginationChanged(pageSize, 1);
  }

  sortBy(key: string) {
    if (this.formatsService.sortOptions.value === key) {
      this.formatsService.reverseSortOrder();
    } else {
      this.formatsService.sortOptions.value = key;
      this.formatsService.sortOptions.order = 'asc';
    }
    this.search();
  }

  get sortOptions() {
    return this.formatsService.sortOptions;
  }

  displayDeletePopup(formatId) {
    const pop = confirm('Etes vous sûr de vouloir supprimer ce format ?');
    if (pop) {
      this.formatsService.delete(formatId).subscribe(() => {
        this.formatsService.pageNumber = 1;
        this.search();
      });
    }
  }
}
