import { Injectable } from '@angular/core';
import { Observable, Subject, from } from 'rxjs';
import { map, concatMap } from 'rxjs/operators';
import { HttpClient } from '@angular/common/http';
import { Resource, IResource, ResourceRO, IResourceFormat } from '../models/resource.model';
import { APP_CONFIG } from './app-config.service';

@Injectable()
export class DataLogsService {

  resourceServiceUrl: string;
  limit: number;
  pageNumber: number;
  sortOptions: {
    value: string,
    order: string,
  };

  private _launchNewAggregationSubject: Subject<any>;

  constructor(
    private _httpClient: HttpClient) {
    this.resourceServiceUrl = `${APP_CONFIG.restHeartAggregations.url}`;
    this._launchNewAggregationSubject = new Subject<any>();
    this.limit = 10;
    this.pageNumber = 1;
  }

  uuidToSessionId(uuid): Observable<any[]> {
    const query = { uuid: uuid };
    const aggregationEndPoint = 'uuidToSessionId?avars=';
    const stringQuery = JSON.stringify(query);
    return this._httpClient.get<any[]>(this.resourceServiceUrl + aggregationEndPoint + stringQuery).pipe(
      map((response) => {
        return response;
      }),
    );
  }

  getAllStepsDuration(uuid, sessionId): Observable<any[]> {
    const query = { uuid: uuid, session_id: sessionId };
    const aggregationEndPoint = 'stepTimeAggr?avars=';
    const stringQuery = JSON.stringify(query);
    return this._httpClient.get<any[]>(this.resourceServiceUrl + aggregationEndPoint + stringQuery).pipe(
      map((response) => {
        return response;
      }),
    );
  }
  getAllUniqueFields(): Observable<any[]> {
    const aggregationEndPoint = 'getAllUniqueFields';
    return this._httpClient.get<any[]>(this.resourceServiceUrl + aggregationEndPoint).pipe(
      map((response) => {
        return response;
      }),
    );
  }
  getData(thisUuid, sessionId): Observable<any[]> {
    const query = { uuid: thisUuid, session_id: sessionId };
    const stringQuery = JSON.stringify(query);
    const aggregationEndPoint = 'reportDataAggr?avars=';
    return this._httpClient.get<any[]>(this.resourceServiceUrl + aggregationEndPoint + stringQuery).pipe(
      map((response) => {
        return response;
      }),
    );
  }
  getProcessTimeAggr(sessionId): Observable<any[]> {
    const query = { session_id: sessionId };
    const stringQuery = JSON.stringify(query);
    const aggregationEndPoint = 'processTimeAggr?avars=';
    return this._httpClient.get<any[]>(this.resourceServiceUrl + aggregationEndPoint + stringQuery).pipe(
      map((response) => {
        return response;
      }),
    );
  }

  getSlugFromSessionId(sessionId): Observable<any[]> {
    const query = { session_id: sessionId };
    const stringQuery = JSON.stringify(query);
    const aggregationEndPoint = 'sessionIdToSlug?avars=';
    return this._httpClient.get<any[]>(this.resourceServiceUrl + aggregationEndPoint + stringQuery).pipe(
      map((response) => {
        return response;
      }),
    );
  }

  getUuidFromSlug(slug): Observable<any[]> {
    const query = { slug };
    const stringQuery = JSON.stringify(query);
    const aggregationEndPoint = 'slugToUuid?avars=';
    return this._httpClient.get<any[]>(this.resourceServiceUrl + aggregationEndPoint + stringQuery).pipe(
      map((response) => {
        return response;
      }),
    );
  }

  getSessionIdFromSlug(slug): Observable<any[]> {
    const query = { slug };
    const stringQuery = JSON.stringify(query);
    const aggregationEndPoint = 'slugToSessionId?avars=';
    return this._httpClient.get<any[]>(this.resourceServiceUrl + aggregationEndPoint + stringQuery).pipe(
      map((response) => {
        return response;
      }),
    );
  }

  getComplementaryFromSlugOrSession(type, typeInfo): Observable<any[]> {
    if (type === 'slug') {
      const query = { slug: typeInfo };
      const stringQuery = JSON.stringify(query);
      const aggregationEndPoint = 'slugToSessionId?avars=';
      return this._httpClient.get<any[]>(this.resourceServiceUrl + aggregationEndPoint + stringQuery).pipe(
        map((response) => {
          return response;
        }),
      );

    } else {
      const query = { session_id: typeInfo };
      const stringQuery = JSON.stringify(query);
      const aggregationEndPoint = 'sessionIdToSlug?avars=';
      return this._httpClient.get<any[]>(this.resourceServiceUrl + aggregationEndPoint + stringQuery).pipe(
        map((response) => {
          return response;
        }),
      );
    }

  }

}
